require File.dirname(__FILE__) + '/test_helper'

module RedmineCrm
  class LiquidTest < ActiveSupport::TestCase
    def test_today_filter
      assert_match Date.today.strftime(date_format), render('{{today}}')
    end

    def test_plus_days_filter
      assert_match (Date.today + 3.days).strftime(date_format), render("{{today | plus_days: 3 | date: '#{date_format}'}}")
    end

    def test_date_range_filter
      assert_equal '10', render("{{today | date_range: '#{Date.today - 10.days}'}}")
    end

    # RedCloth3 is a part of Redmine - test will not work
    # def test_textilize_filter
    #   assert_match '<h1>Some text</h1>', render("{{ 'Some text' | textilize }}")
    # end

    def test_underscore_filter
      assert_match 'some_text', render("{{ 'some text' | underscore }}")
    end

    def test_dasherize_filter
      assert_match 'some-text', render("{{ 'some text' | dasherize }}")
    end

    def test_random_filter
      assert render("{{ random: 10 }}").to_i <= 10
    end

    def test_user_drop
      user = User.first
      assert_match user.name, render("{{ user.name }}")
    end

    def test_users_drop
      users_name = User.all.map(&:name)
      rendered_text = render("{% for user in users %} {{user.name }} {% endfor %}")
      users_name.each do |name|
        assert_match name, rendered_text
      end
    end

    def test_issue_drop
      assert_match Issue.first.description, render("{{ issue.description }}")
    end

    def test_default_filter
      user = User.first
      user.name = nil
      user.save
      assert_match 'user', render("{{ user.name | default: 'user' }}")
    end

    private

    def render(content)
      assigns = {}
      assigns['now'] = Time.now
      assigns['today'] = Date.today.strftime(date_format)
      assigns['user'] = Liquid::UserDrop.new(User.first)
      assigns['users'] = Liquid::UsersDrop.new(User.all)
      assigns['issue'] = Liquid::IssueDrop.new(Issue.first)

      registers = {}
      begin
        ::Liquid::Template.parse(content).render(::Liquid::Context.new({}, assigns, registers)).html_safe
      rescue => e
        e.message
      end
    end

    def date_format
      '%d.%m.%Y'
    end
  end
end
